#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <string.h>
#include "ff.h"
#include "diskio.h"
#include "1wire.h"
#include "lcd.h"
#include "keyb.h"
#include "i2c.h"
#include "MCP79410.h"

#define DS18B20_CONVERT_T 0x44
#define DS18B20_READ_SCRATCHPAD 0xBE

typedef struct {
	DateTime dateTime;
	float temperature;
} TemperatureRecord;

enum TemperatureState {
	TEMP_IDLE,
	TEMP_IN_PROGRESS,
	TEMP_READY
};

void getTemperature(float * temperature);
void setDateTime(void);
void displayDateTime(void);
void displayRecord(TemperatureRecord * temperatureRecord);
void getSeparatedNumbers(char str[], uint16_t outArray[], uint8_t outArraySize);
void writeRecord(TemperatureRecord * temperatureRecord);

void SDTimerInit(void) {
	OCR0 = F_CPU / 1024 / 100 - 1;
	TCCR0 = _BV(WGM01) | _BV(CS02) | _BV(CS00); //CTC, F_CPU/1024
	TIMSK = _BV(OCIE0); //output compare
}

ISR(TIMER0_COMP_vect) {
	disk_timerproc();
}

int main(void) {
	i2cInit();
	keybInit();
	lcdInit();
	lcdInitPrintf();
	SDTimerInit(); //timer do komunikacji z kart
	sei();
	uint8_t recordWritten = 0;
	TemperatureRecord temperatureRecord;
	while(1){
		//odczyt temperatury
		getTemperature(&temperatureRecord.temperature);
		//odczyt daty i czasu
		DateTime * dateTime = &temperatureRecord.dateTime;
		rtcReadDateTime(dateTime);
		//zapis rekordu
		if ((dateTime->seconds % 15) == 0) {
			if (!recordWritten) {
				writeRecord(&temperatureRecord);
				recordWritten = 1;
			}
		} else {
			recordWritten = 0;
		}
		//wywietlenie danych
		displayRecord(&temperatureRecord);
		//obsuga klawiatury
		if (readKeyboard()) {
			uint8_t key = getKey();
			if (key == 1) setDateTime();
		}
	}
}

void writeRecord(TemperatureRecord * temperatureRecord) {
	FATFS fatFs;
	FRESULT fresult = f_mount(&fatFs, "", 1);
	if(fresult == FR_OK) {
		FIL fp;
		fresult = f_open(&fp, "temper.csv", FA_WRITE | FA_OPEN_APPEND);
		char buff[16];
		DateTime * dateTime = &temperatureRecord->dateTime;
		UINT l = sprintf(buff, "%02d:%02d:%02d ", dateTime->hours, dateTime->minutes, dateTime->seconds);
		f_write(&fp, buff, l, &l);
		l = sprintf(buff, "%02d-%02d-%4d", dateTime->dayOfMonth, dateTime->month, dateTime->year);
		f_write(&fp, buff, l, &l);
		l = sprintf(buff, "; %.2f\r\n", temperatureRecord->temperature);
		for (uint8_t i = 0; i < l; i++) if (buff[i] == '.') buff[i] = ',';
		f_write(&fp, buff, l, &l);
		f_close(&fp);
		lcdGotoXY(11, 1);
		printf("SD OK");
	} else {
		lcdGotoXY(11, 1);
		printf("SDERR");
	}
}

void getSeparatedNumbers(char str[], uint16_t outArray[], uint8_t outArraySize) {
	uint8_t strPos = 0;
	uint8_t outPos = 0;
	for (uint8_t i = 0; i < outArraySize; i++) outArray[i] = 0;
	while ((str[strPos] != 0) && (outPos < outArraySize)) {
		if (str[strPos] < '0' || str[strPos] > '9') {
			outPos++;
		} else {
			outArray[outPos] = outArray[outPos] * 10 + (str[strPos] - '0');
		}
		strPos++;
	}
}

void displayRecord(TemperatureRecord * temperatureRecord) {
	DateTime * dateTime = &temperatureRecord->dateTime;
	lcdGotoXY(0, 0);
	printf("%02d:%02d:%02d", dateTime->hours, dateTime->minutes, dateTime->seconds);
	lcdGotoXY(9, 0);
	printf("t=%.1f\xDF", temperatureRecord->temperature);
	lcdGotoXY(0, 1);
	printf("%02d-%02d-%4d", dateTime->dayOfMonth, dateTime->month, dateTime->year);
}

void getTemperature(float * temperature) {
	static enum TemperatureState temperatureState = TEMP_IDLE;
	switch (temperatureState) {
		case TEMP_IDLE:
			oneWireReset();
			oneWireSendByte(OW_SKIP_ROM);
			oneWireSendByte(DS18B20_CONVERT_T);
			temperatureState = TEMP_IN_PROGRESS;
			break;
		case TEMP_IN_PROGRESS:
			if (oneWireReceiveBit()) temperatureState = TEMP_READY;
			break;
		case TEMP_READY:
			oneWireReset();
			oneWireSendByte(OW_SKIP_ROM);
			oneWireSendByte(DS18B20_READ_SCRATCHPAD);
			int16_t temp;
			uint8_t * byte = (uint8_t *) &temp;
			byte[0] = oneWireReceiveByte();
			byte[1] = oneWireReceiveByte();
			*temperature = temp / 16.0;
			temperatureState = TEMP_IDLE;
			break;
	}
}

void setDateTime(void) {
	//wcz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_CURSOR | LCD_PARAM_ON_OFF_DISPLAY);
	//wczytaj czas
	lcdGotoXY(0, 0);
	printf("  :  :  ");
	lcdGotoXY(0, 0);
	const uint8_t timeFieldsSizes[] = {2, 2, 2};
	uint16_t time[3];
	readSeparatedNumbers(timeFieldsSizes, 3, time);
	//wczytaj dat
	lcdGotoXY(0, 1);
	printf("  -  -    ");
	lcdGotoXY(0, 1);
	const uint8_t dateFieldsSizes[] = {2, 2, 4};
	uint16_t date[3];
	readSeparatedNumbers(dateFieldsSizes, 3, date);
	DateTime newDateTime;
	newDateTime.hours = time[0];
	newDateTime.minutes = time[1];
	newDateTime.seconds = time[2];
	newDateTime.dayOfMonth = date[0];
	newDateTime.month = date[1];
	newDateTime.year = date[2];
	//zapisz nowy czas i dat
	rtcWriteDateTime(&newDateTime);
	//wycz kursor
	lcdWriteCommand(LCD_COMMAND_ON_OFF | LCD_PARAM_ON_OFF_DISPLAY);
}
